import configparser
import secrets

from core.models import UserAccount as User
from core.models import Reviewer, Student, Tutor

STUDENTS = 'students'
TUTORS = 'tutors'
REVIEWERS = 'reviewers'

PASSWORDS = '.importer_passwords'


def get_xkcd_password(k=2):
    with open('/usr/share/dict/words') as words:
        choose_from = list({word.strip().lower()
                            for word in words if 5 < len(word) < 8})

    return ''.join(secrets.choice(choose_from) for _ in range(k))


def store_password(username, groupname, password):
    storage = configparser.ConfigParser()
    storage.read(PASSWORDS)

    if groupname not in storage:
        storage[groupname] = {}

    storage[groupname][username] = password

    with open(PASSWORDS, 'w') as passwd_file:
        storage.write(passwd_file)


class GradyUserFactory:

    def __init__(self,
                 password_generator_func=get_xkcd_password,
                 password_storge=store_password,
                 *args, **kwargs):
        self.password_generator_func = password_generator_func
        self.password_storge = password_storge

    @staticmethod
    def _get_random_name(prefix='', suffix='', k=1):
        return ''.join((prefix, get_xkcd_password(k), suffix))

    def _make_base_user(self, username, groupname, store_pw=False, **kwargs):
        """ This is a specific wrapper for the django update_or_create method of
        objects.
            * A new user is created and password and group are set accordingly
            * If the user was there before password is NOT change but group is. A
              user must only have one group.

        Returns:
            (User object, str): The user object that was added to the group and
            the password of that user if it was created.
        """
        username = username.strip()

        user, created = User.objects.update_or_create(
            username=username,
            defaults=kwargs)

        if created:
            password = self.password_generator_func()
            user.set_password(password)
            user.save()

        if created and store_pw:
            self.password_storge(username, groupname, password)

        return user

    def _get_user_model_for_group(self, groupname):
        """ Returns the model class for a usergroup """
        return {
            STUDENTS: Student,
            TUTORS: Tutor,
            REVIEWERS: Reviewer,
        }[groupname]

    def _make_user_generic(self, username, groupname, **kwargs):
        """ Provides a model with a associated user but without any defaults
        """

        if not username:
            username = self._get_random_name(prefix=groupname.lower() + '_')

        model = self._get_user_model_for_group(groupname)
        user = self._make_base_user(username, groupname, **kwargs)

        generic_user, _ = model.objects.get_or_create(user=user)

        return generic_user

    def make_student(self, username=None, matrikel_no=None, exam=None, **kwargs):
        """ Creates a student. Defaults can be passed via kwargs like in
        relation managers objects.update method. """
        user = self._make_user_generic(username, STUDENTS, **kwargs)
        if matrikel_no:
            user.objects.update(matrikel_no=matrikel_no)
        if exam:
            user.objects.update(exam=exam)
        return user

    def make_tutor(self, username=None, **kwargs):
        """ Creates or updates a tutor if needed with defaults """
        return self._make_user_generic(username, TUTORS, **kwargs)

    def make_reviewer(self, username=None, **kwargs):
        """ Creates or updates a reviewer if needed with defaults """
        return self._make_user_generic(username, REVIEWERS, **kwargs)
